<?php

namespace Fastrack\Utilities;

use PlatoCreative\Imgix\Imgix;
use Psr\Container\NotFoundExceptionInterface;
use Psr\SimpleCache\CacheInterface;
use Psr\SimpleCache\InvalidArgumentException;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Control\HTTPResponse;
use SilverStripe\Control\Middleware\HTTPCacheControlMiddleware;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\Security\Security;

class Api
{

	public static function constructResponse(Controller $controller, array $data): HTTPResponse
	{
		HTTPCacheControlMiddleware::singleton()->enableCache(true, 60 * 60);
		$controller->getResponse()->setBody(json_encode($data));
		$controller->getResponse()->addHeader("Content-type", "application/json");
		return $controller->getResponse();
	}


	public static function fetchFromCache($key, $default)
	{
		if (!Api::shouldCache()) return $default;
		try {
			return Api::getCache()->get($key, $default);
		} catch (NotFoundExceptionInterface|InvalidArgumentException $e) {
			return $default;
		}
	}

	public static function setCache(string $key, $value) {
		if (!Api::shouldCache()) return;
		try {
			Api::getCache()->set($key, $value, 60 * 60 * 6);
		} catch (NotFoundExceptionInterface|InvalidArgumentException $e) {
			user_error($e->getMessage(), E_USER_WARNING);
		}
	}

	/**
	 * @throws NotFoundExceptionInterface
	 */
	public static function getCache(): CacheInterface
	{
		return Injector::inst()->get(CacheInterface::class . '.fastrackApiCache');
	}

	public static function shouldCache(): bool
	{
		if (Security::getCurrentUser()) {
			return false;
		}
		if (Director::isDev()) {
			return false;
		}
		return true;
	}

	public static function buildImage(?Imgix $image): ?array
	{
		if (!$image->ID) {
			return null;
		}
		$scale = $image->useImgix();
		return [
			'alt' => $image->Title,
			'md' => $scale ? $image->ScaleMaxWidth(992)->Link() : $image->AbsoluteLink(),
			'lg' => $scale ? $image->ScaleMaxWidth(1920)->Link() : $image->AbsoluteLink(),
		];
	}

	public static function buildGallery($model, string $galleryKey, int $maxImages = 4): array
	{
		$slides = [];
		$gallery = $model->Galleries()->find('Key', $galleryKey);
		if ($gallery) {
			foreach ($gallery->Slides() as $slide) {
				if (count($slides) === $maxImages) {
					break;
				}
				$image = Api::buildImage($slide->Image());
				if ($image) {
					$slides[] = [
						'title' => $slide->Title,
						'image' => Api::buildImage($slide->Image()),
					];
				}
			}
		}
		return $slides;
	}

}
